# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .scans import (
    ScansResource,
    AsyncScansResource,
    ScansResourceWithRawResponse,
    AsyncScansResourceWithRawResponse,
    ScansResourceWithStreamingResponse,
    AsyncScansResourceWithStreamingResponse,
)
from .responses import (
    ResponsesResource,
    AsyncResponsesResource,
    ResponsesResourceWithRawResponse,
    AsyncResponsesResourceWithRawResponse,
    ResponsesResourceWithStreamingResponse,
    AsyncResponsesResourceWithStreamingResponse,
)
from .url_scanner import (
    URLScannerResource,
    AsyncURLScannerResource,
    URLScannerResourceWithRawResponse,
    AsyncURLScannerResourceWithRawResponse,
    URLScannerResourceWithStreamingResponse,
    AsyncURLScannerResourceWithStreamingResponse,
)

__all__ = [
    "ResponsesResource",
    "AsyncResponsesResource",
    "ResponsesResourceWithRawResponse",
    "AsyncResponsesResourceWithRawResponse",
    "ResponsesResourceWithStreamingResponse",
    "AsyncResponsesResourceWithStreamingResponse",
    "ScansResource",
    "AsyncScansResource",
    "ScansResourceWithRawResponse",
    "AsyncScansResourceWithRawResponse",
    "ScansResourceWithStreamingResponse",
    "AsyncScansResourceWithStreamingResponse",
    "URLScannerResource",
    "AsyncURLScannerResource",
    "URLScannerResourceWithRawResponse",
    "AsyncURLScannerResourceWithRawResponse",
    "URLScannerResourceWithStreamingResponse",
    "AsyncURLScannerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ......_types import Body, Query, Headers, NotGiven, not_given
from ......_utils import path_template, maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......pagination import SyncSinglePage, AsyncSinglePage
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.devices.fallback_domain import FallbackDomain
from ......types.zero_trust.devices.fallback_domain_param import FallbackDomainParam

__all__ = ["FallbackDomainsResource", "AsyncFallbackDomainsResource"]


class FallbackDomainsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FallbackDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FallbackDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FallbackDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FallbackDomainsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        domains: Iterable[FallbackDomainParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[FallbackDomain]:
        """Sets the list of domains to bypass Gateway DNS resolution.

        These domains will
        use the specified local DNS resolver instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/policy/fallback_domains", account_id=account_id),
            page=SyncSinglePage[FallbackDomain],
            body=maybe_transform(domains, Iterable[FallbackDomainParam]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
            method="put",
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[FallbackDomain]:
        """Fetches a list of domains to bypass Gateway DNS resolution.

        These domains will
        use the specified local DNS resolver instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/policy/fallback_domains", account_id=account_id),
            page=SyncSinglePage[FallbackDomain],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
        )


class AsyncFallbackDomainsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFallbackDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFallbackDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFallbackDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFallbackDomainsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        domains: Iterable[FallbackDomainParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FallbackDomain, AsyncSinglePage[FallbackDomain]]:
        """Sets the list of domains to bypass Gateway DNS resolution.

        These domains will
        use the specified local DNS resolver instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/policy/fallback_domains", account_id=account_id),
            page=AsyncSinglePage[FallbackDomain],
            body=maybe_transform(domains, Iterable[FallbackDomainParam]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
            method="put",
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FallbackDomain, AsyncSinglePage[FallbackDomain]]:
        """Fetches a list of domains to bypass Gateway DNS resolution.

        These domains will
        use the specified local DNS resolver instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/devices/policy/fallback_domains", account_id=account_id),
            page=AsyncSinglePage[FallbackDomain],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
        )


class FallbackDomainsResourceWithRawResponse:
    def __init__(self, fallback_domains: FallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = to_raw_response_wrapper(
            fallback_domains.update,
        )
        self.get = to_raw_response_wrapper(
            fallback_domains.get,
        )


class AsyncFallbackDomainsResourceWithRawResponse:
    def __init__(self, fallback_domains: AsyncFallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = async_to_raw_response_wrapper(
            fallback_domains.update,
        )
        self.get = async_to_raw_response_wrapper(
            fallback_domains.get,
        )


class FallbackDomainsResourceWithStreamingResponse:
    def __init__(self, fallback_domains: FallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = to_streamed_response_wrapper(
            fallback_domains.update,
        )
        self.get = to_streamed_response_wrapper(
            fallback_domains.get,
        )


class AsyncFallbackDomainsResourceWithStreamingResponse:
    def __init__(self, fallback_domains: AsyncFallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = async_to_streamed_response_wrapper(
            fallback_domains.update,
        )
        self.get = async_to_streamed_response_wrapper(
            fallback_domains.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .keyless_certificates import (
    KeylessCertificatesResource,
    AsyncKeylessCertificatesResource,
    KeylessCertificatesResourceWithRawResponse,
    AsyncKeylessCertificatesResourceWithRawResponse,
    KeylessCertificatesResourceWithStreamingResponse,
    AsyncKeylessCertificatesResourceWithStreamingResponse,
)

__all__ = [
    "KeylessCertificatesResource",
    "AsyncKeylessCertificatesResource",
    "KeylessCertificatesResourceWithRawResponse",
    "AsyncKeylessCertificatesResourceWithRawResponse",
    "KeylessCertificatesResourceWithStreamingResponse",
    "AsyncKeylessCertificatesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.brand_protection import logo_match_get_params, logo_match_download_params
from ...types.brand_protection.logo_match_get_response import LogoMatchGetResponse
from ...types.brand_protection.logo_match_download_response import LogoMatchDownloadResponse

__all__ = ["LogoMatchesResource", "AsyncLogoMatchesResource"]


class LogoMatchesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LogoMatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogoMatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogoMatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogoMatchesResourceWithStreamingResponse(self)

    def download(
        self,
        *,
        account_id: str,
        limit: str | Omit = omit,
        logo_id: SequenceNotStr[str] | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoMatchDownloadResponse:
        """
        Return matches as CSV for logo queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/brand-protection/logo-matches/download", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "logo_id": logo_id,
                        "offset": offset,
                    },
                    logo_match_download_params.LogoMatchDownloadParams,
                ),
            ),
            cast_to=LogoMatchDownloadResponse,
        )

    def get(
        self,
        *,
        account_id: str,
        limit: str | Omit = omit,
        logo_id: SequenceNotStr[str] | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoMatchGetResponse:
        """
        Return matches for logo queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/brand-protection/logo-matches", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "logo_id": logo_id,
                        "offset": offset,
                    },
                    logo_match_get_params.LogoMatchGetParams,
                ),
            ),
            cast_to=LogoMatchGetResponse,
        )


class AsyncLogoMatchesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLogoMatchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogoMatchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogoMatchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogoMatchesResourceWithStreamingResponse(self)

    async def download(
        self,
        *,
        account_id: str,
        limit: str | Omit = omit,
        logo_id: SequenceNotStr[str] | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoMatchDownloadResponse:
        """
        Return matches as CSV for logo queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/brand-protection/logo-matches/download", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "logo_id": logo_id,
                        "offset": offset,
                    },
                    logo_match_download_params.LogoMatchDownloadParams,
                ),
            ),
            cast_to=LogoMatchDownloadResponse,
        )

    async def get(
        self,
        *,
        account_id: str,
        limit: str | Omit = omit,
        logo_id: SequenceNotStr[str] | Omit = omit,
        offset: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LogoMatchGetResponse:
        """
        Return matches for logo queries based on ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/brand-protection/logo-matches", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "logo_id": logo_id,
                        "offset": offset,
                    },
                    logo_match_get_params.LogoMatchGetParams,
                ),
            ),
            cast_to=LogoMatchGetResponse,
        )


class LogoMatchesResourceWithRawResponse:
    def __init__(self, logo_matches: LogoMatchesResource) -> None:
        self._logo_matches = logo_matches

        self.download = to_raw_response_wrapper(
            logo_matches.download,
        )
        self.get = to_raw_response_wrapper(
            logo_matches.get,
        )


class AsyncLogoMatchesResourceWithRawResponse:
    def __init__(self, logo_matches: AsyncLogoMatchesResource) -> None:
        self._logo_matches = logo_matches

        self.download = async_to_raw_response_wrapper(
            logo_matches.download,
        )
        self.get = async_to_raw_response_wrapper(
            logo_matches.get,
        )


class LogoMatchesResourceWithStreamingResponse:
    def __init__(self, logo_matches: LogoMatchesResource) -> None:
        self._logo_matches = logo_matches

        self.download = to_streamed_response_wrapper(
            logo_matches.download,
        )
        self.get = to_streamed_response_wrapper(
            logo_matches.get,
        )


class AsyncLogoMatchesResourceWithStreamingResponse:
    def __init__(self, logo_matches: AsyncLogoMatchesResource) -> None:
        self._logo_matches = logo_matches

        self.download = async_to_streamed_response_wrapper(
            logo_matches.download,
        )
        self.get = async_to_streamed_response_wrapper(
            logo_matches.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Item"]


class Item(BaseModel):
    id: str
    """UUID."""

    content: str
    """Request content."""

    created: datetime

    priority: datetime

    request: str
    """Requested information from request."""

    summary: str
    """Brief description of the request."""

    tlp: Literal["clear", "amber", "amber-strict", "green", "red"]
    """The CISA defined Traffic Light Protocol (TLP)."""

    updated: datetime

    completed: Optional[datetime] = None

    message_tokens: Optional[int] = None
    """Tokens for the request messages."""

    readable_id: Optional[str] = None
    """Readable Request ID."""

    status: Optional[Literal["open", "accepted", "reported", "approved", "completed", "declined"]] = None
    """Request Status."""

    tokens: Optional[int] = None
    """Tokens for the request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "ConsumerCreateParams",
    "MqWorkerConsumerRequest",
    "MqWorkerConsumerRequestSettings",
    "MqHTTPConsumerRequest",
    "MqHTTPConsumerRequestSettings",
]


class MqWorkerConsumerRequest(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    script_name: Required[str]
    """Name of a Worker"""

    type: Required[Literal["worker"]]

    dead_letter_queue: str

    settings: MqWorkerConsumerRequestSettings


class MqWorkerConsumerRequestSettings(TypedDict, total=False):
    batch_size: float
    """The maximum number of messages to include in a batch."""

    max_concurrency: float
    """Maximum number of concurrent consumers that may consume from this Queue.

    Set to `null` to automatically opt in to the platform's maximum (recommended).
    """

    max_retries: float
    """The maximum number of retries"""

    max_wait_time_ms: float
    """
    The number of milliseconds to wait for a batch to fill up before attempting to
    deliver it
    """

    retry_delay: float
    """
    The number of seconds to delay before making the message available for another
    attempt.
    """


class MqHTTPConsumerRequest(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    type: Required[Literal["http_pull"]]

    dead_letter_queue: str

    settings: MqHTTPConsumerRequestSettings


class MqHTTPConsumerRequestSettings(TypedDict, total=False):
    batch_size: float
    """The maximum number of messages to include in a batch."""

    max_retries: float
    """The maximum number of retries"""

    retry_delay: float
    """
    The number of seconds to delay before making the message available for another
    attempt.
    """

    visibility_timeout_ms: float
    """The number of milliseconds that a message is exclusively leased.

    After the timeout, the message becomes available for another attempt.
    """


ConsumerCreateParams: TypeAlias = Union[MqWorkerConsumerRequest, MqHTTPConsumerRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .fraud_settings import FraudSettings as FraudSettings
from .fraud_update_params import FraudUpdateParams as FraudUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MaintenanceConfigUpdateParams", "Compaction", "SnapshotExpiration"]


class MaintenanceConfigUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Use this to identify the account."""

    bucket_name: Required[str]
    """Specifies the R2 bucket name."""

    namespace: Required[str]

    compaction: Compaction
    """Updates compaction configuration (all fields optional)."""

    snapshot_expiration: SnapshotExpiration
    """Updates snapshot expiration configuration (all fields optional)."""


class Compaction(TypedDict, total=False):
    """Updates compaction configuration (all fields optional)."""

    state: Literal["enabled", "disabled"]
    """Updates the state optionally."""

    target_size_mb: Literal["64", "128", "256", "512"]
    """Updates the target file size optionally."""


class SnapshotExpiration(TypedDict, total=False):
    """Updates snapshot expiration configuration (all fields optional)."""

    max_snapshot_age: str
    """Updates the maximum age for snapshots optionally."""

    min_snapshots_to_keep: int
    """Updates the minimum number of snapshots to retain optionally."""

    state: Literal["enabled", "disabled"]
    """Updates the state optionally."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from .health_check import HealthCheck

__all__ = ["CfInterconnectBulkUpdateResponse", "ModifiedInterconnect", "ModifiedInterconnectGRE"]


class ModifiedInterconnectGRE(BaseModel):
    """The configuration specific to GRE interconnects."""

    cloudflare_endpoint: Optional[str] = None
    """
    The IP address assigned to the Cloudflare side of the GRE tunnel created as part
    of the Interconnect.
    """


class ModifiedInterconnect(BaseModel):
    id: Optional[str] = None
    """Identifier"""

    automatic_return_routing: Optional[bool] = None
    """
    True if automatic stateful return routing should be enabled for a tunnel, false
    otherwise. Requires the `coupler_integration` account flag to be enabled;
    requests setting this to `true` without that flag will be rejected.
    """

    colo_name: Optional[str] = None
    """The name of the interconnect. The name cannot share a name with other tunnels."""

    created_on: Optional[datetime] = None
    """The date and time the tunnel was created."""

    description: Optional[str] = None
    """An optional description of the interconnect."""

    gre: Optional[ModifiedInterconnectGRE] = None
    """The configuration specific to GRE interconnects."""

    health_check: Optional[HealthCheck] = None

    interface_address: Optional[str] = None
    """
    A 31-bit prefix (/31 in CIDR notation) supporting two hosts, one for each side
    of the tunnel. Select the subnet from the following private IP space:
    10.0.0.0–10.255.255.255, 172.16.0.0–172.31.255.255, 192.168.0.0–192.168.255.255.
    """

    interface_address6: Optional[str] = None
    """
    A 127 bit IPV6 prefix from within the virtual_subnet6 prefix space with the
    address being the first IP of the subnet and not same as the address of
    virtual_subnet6. Eg if virtual_subnet6 is 2606:54c1:7:0:a9fe:12d2::/127 ,
    interface_address6 could be 2606:54c1:7:0:a9fe:12d2:1:200/127
    """

    modified_on: Optional[datetime] = None
    """The date and time the tunnel was last modified."""

    mtu: Optional[int] = None
    """The Maximum Transmission Unit (MTU) in bytes for the interconnect.

    The minimum value is 576.
    """

    name: Optional[str] = None
    """The name of the interconnect. The name cannot share a name with other tunnels."""

    virtual_port_reservation_id: Optional[str] = None
    """
    An identifier that correlates this interconnect with the corresponding V2 CNI
    interconnect resource.
    """


class CfInterconnectBulkUpdateResponse(BaseModel):
    modified: Optional[bool] = None

    modified_interconnects: Optional[List[ModifiedInterconnect]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from .web_crawlers import (
    WebCrawlersResource,
    AsyncWebCrawlersResource,
    WebCrawlersResourceWithRawResponse,
    AsyncWebCrawlersResourceWithRawResponse,
    WebCrawlersResourceWithStreamingResponse,
    AsyncWebCrawlersResourceWithStreamingResponse,
)
from ....types.radar import (
    bot_get_params,
    bot_list_params,
    bot_summary_params,
    bot_timeseries_params,
    bot_timeseries_groups_params,
)
from ...._base_client import make_request_options
from ....types.radar.bot_get_response import BotGetResponse
from ....types.radar.bot_list_response import BotListResponse
from ....types.radar.bot_summary_response import BotSummaryResponse
from ....types.radar.bot_timeseries_response import BotTimeseriesResponse
from ....types.radar.bot_timeseries_groups_response import BotTimeseriesGroupsResponse

__all__ = ["BotsResource", "AsyncBotsResource"]


class BotsResource(SyncAPIResource):
    @cached_property
    def web_crawlers(self) -> WebCrawlersResource:
        return WebCrawlersResource(self._client)

    @cached_property
    def with_raw_response(self) -> BotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BotsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        bot_category: Literal[
            "SEARCH_ENGINE_CRAWLER",
            "SEARCH_ENGINE_OPTIMIZATION",
            "MONITORING_AND_ANALYTICS",
            "ADVERTISING_AND_MARKETING",
            "SOCIAL_MEDIA_MARKETING",
            "PAGE_PREVIEW",
            "ACADEMIC_RESEARCH",
            "SECURITY",
            "ACCESSIBILITY",
            "WEBHOOKS",
            "FEED_FETCHER",
            "AI_CRAWLER",
            "AGGREGATOR",
            "AI_ASSISTANT",
            "AI_SEARCH",
            "ARCHIVER",
        ]
        | Omit = omit,
        bot_operator: str | Omit = omit,
        bot_verification_status: Literal["VERIFIED"] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        kind: Literal["AGENT", "BOT"] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotListResponse:
        """
        Retrieves a list of bots.

        Args:
          bot_category: Filters results by bot category.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status.

          format: Format in which results will be returned.

          kind: Filters results by bot kind.

          limit: Limits the number of objects returned in the response.

          offset: Skips the specified number of objects before fetching the results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bots",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "bot_category": bot_category,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "format": format,
                        "kind": kind,
                        "limit": limit,
                        "offset": offset,
                    },
                    bot_list_params.BotListParams,
                ),
                post_parser=ResultWrapper[BotListResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotListResponse], ResultWrapper[BotListResponse]),
        )

    def get(
        self,
        bot_slug: str,
        *,
        format: Literal["JSON", "CSV"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotGetResponse:
        """
        Retrieves the requested bot information.

        Args:
          bot_slug: Bot slug.

          format: Format in which results will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bot_slug:
            raise ValueError(f"Expected a non-empty value for `bot_slug` but received {bot_slug!r}")
        return self._get(
            path_template("/radar/bots/{bot_slug}", bot_slug=bot_slug),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"format": format}, bot_get_params.BotGetParams),
                post_parser=ResultWrapper[BotGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotGetResponse], ResultWrapper[BotGetResponse]),
        )

    def summary(
        self,
        dimension: Literal["BOT", "BOT_KIND", "BOT_OPERATOR", "BOT_CATEGORY"],
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        bot: SequenceNotStr[str] | Omit = omit,
        bot_category: List[
            Literal[
                "SEARCH_ENGINE_CRAWLER",
                "SEARCH_ENGINE_OPTIMIZATION",
                "MONITORING_AND_ANALYTICS",
                "ADVERTISING_AND_MARKETING",
                "SOCIAL_MEDIA_MARKETING",
                "PAGE_PREVIEW",
                "ACADEMIC_RESEARCH",
                "SECURITY",
                "ACCESSIBILITY",
                "WEBHOOKS",
                "FEED_FETCHER",
                "AI_CRAWLER",
                "AGGREGATOR",
                "AI_ASSISTANT",
                "AI_SEARCH",
                "ARCHIVER",
            ]
        ]
        | Omit = omit,
        bot_kind: List[Literal["AGENT", "BOT"]] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        bot_verification_status: List[Literal["VERIFIED"]] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotSummaryResponse:
        """
        Retrieves an aggregated summary of bots HTTP requests grouped by the specified
        dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bot: Filters results by bot name.

          bot_category: Filters results by bot category.

          bot_kind: Filters results by bot kind.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status (Verified vs. Unverified).

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/bots/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "asn": asn,
                        "bot": bot,
                        "bot_category": bot_category,
                        "bot_kind": bot_kind,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    bot_summary_params.BotSummaryParams,
                ),
                post_parser=ResultWrapper[BotSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotSummaryResponse], ResultWrapper[BotSummaryResponse]),
        )

    def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        bot: SequenceNotStr[str] | Omit = omit,
        bot_category: List[
            Literal[
                "SEARCH_ENGINE_CRAWLER",
                "SEARCH_ENGINE_OPTIMIZATION",
                "MONITORING_AND_ANALYTICS",
                "ADVERTISING_AND_MARKETING",
                "SOCIAL_MEDIA_MARKETING",
                "PAGE_PREVIEW",
                "ACADEMIC_RESEARCH",
                "SECURITY",
                "ACCESSIBILITY",
                "WEBHOOKS",
                "FEED_FETCHER",
                "AI_CRAWLER",
                "AGGREGATOR",
                "AI_ASSISTANT",
                "AI_SEARCH",
                "ARCHIVER",
            ]
        ]
        | Omit = omit,
        bot_kind: List[Literal["AGENT", "BOT"]] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        bot_verification_status: List[Literal["VERIFIED"]] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotTimeseriesResponse:
        """
        Retrieves bots HTTP request volume over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bot: Filters results by bot name.

          bot_category: Filters results by bot category.

          bot_kind: Filters results by bot kind.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status (Verified vs. Unverified).

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/bots/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "bot": bot,
                        "bot_category": bot_category,
                        "bot_kind": bot_kind,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "name": name,
                    },
                    bot_timeseries_params.BotTimeseriesParams,
                ),
                post_parser=ResultWrapper[BotTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotTimeseriesResponse], ResultWrapper[BotTimeseriesResponse]),
        )

    def timeseries_groups(
        self,
        dimension: Literal["BOT", "BOT_KIND", "BOT_OPERATOR", "BOT_CATEGORY"],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        bot: SequenceNotStr[str] | Omit = omit,
        bot_category: List[
            Literal[
                "SEARCH_ENGINE_CRAWLER",
                "SEARCH_ENGINE_OPTIMIZATION",
                "MONITORING_AND_ANALYTICS",
                "ADVERTISING_AND_MARKETING",
                "SOCIAL_MEDIA_MARKETING",
                "PAGE_PREVIEW",
                "ACADEMIC_RESEARCH",
                "SECURITY",
                "ACCESSIBILITY",
                "WEBHOOKS",
                "FEED_FETCHER",
                "AI_CRAWLER",
                "AGGREGATOR",
                "AI_ASSISTANT",
                "AI_SEARCH",
                "ARCHIVER",
            ]
        ]
        | Omit = omit,
        bot_kind: List[Literal["AGENT", "BOT"]] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        bot_verification_status: List[Literal["VERIFIED"]] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotTimeseriesGroupsResponse:
        """
        Retrieves the distribution of HTTP requests from bots, grouped by the specified
        dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bot: Filters results by bot name.

          bot_category: Filters results by bot category.

          bot_kind: Filters results by bot kind.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status (Verified vs. Unverified).

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/bots/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "bot": bot,
                        "bot_category": bot_category,
                        "bot_kind": bot_kind,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    bot_timeseries_groups_params.BotTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[BotTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotTimeseriesGroupsResponse], ResultWrapper[BotTimeseriesGroupsResponse]),
        )


class AsyncBotsResource(AsyncAPIResource):
    @cached_property
    def web_crawlers(self) -> AsyncWebCrawlersResource:
        return AsyncWebCrawlersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBotsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        bot_category: Literal[
            "SEARCH_ENGINE_CRAWLER",
            "SEARCH_ENGINE_OPTIMIZATION",
            "MONITORING_AND_ANALYTICS",
            "ADVERTISING_AND_MARKETING",
            "SOCIAL_MEDIA_MARKETING",
            "PAGE_PREVIEW",
            "ACADEMIC_RESEARCH",
            "SECURITY",
            "ACCESSIBILITY",
            "WEBHOOKS",
            "FEED_FETCHER",
            "AI_CRAWLER",
            "AGGREGATOR",
            "AI_ASSISTANT",
            "AI_SEARCH",
            "ARCHIVER",
        ]
        | Omit = omit,
        bot_operator: str | Omit = omit,
        bot_verification_status: Literal["VERIFIED"] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        kind: Literal["AGENT", "BOT"] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotListResponse:
        """
        Retrieves a list of bots.

        Args:
          bot_category: Filters results by bot category.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status.

          format: Format in which results will be returned.

          kind: Filters results by bot kind.

          limit: Limits the number of objects returned in the response.

          offset: Skips the specified number of objects before fetching the results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bots",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "bot_category": bot_category,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "format": format,
                        "kind": kind,
                        "limit": limit,
                        "offset": offset,
                    },
                    bot_list_params.BotListParams,
                ),
                post_parser=ResultWrapper[BotListResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotListResponse], ResultWrapper[BotListResponse]),
        )

    async def get(
        self,
        bot_slug: str,
        *,
        format: Literal["JSON", "CSV"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotGetResponse:
        """
        Retrieves the requested bot information.

        Args:
          bot_slug: Bot slug.

          format: Format in which results will be returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bot_slug:
            raise ValueError(f"Expected a non-empty value for `bot_slug` but received {bot_slug!r}")
        return await self._get(
            path_template("/radar/bots/{bot_slug}", bot_slug=bot_slug),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"format": format}, bot_get_params.BotGetParams),
                post_parser=ResultWrapper[BotGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotGetResponse], ResultWrapper[BotGetResponse]),
        )

    async def summary(
        self,
        dimension: Literal["BOT", "BOT_KIND", "BOT_OPERATOR", "BOT_CATEGORY"],
        *,
        asn: SequenceNotStr[str] | Omit = omit,
        bot: SequenceNotStr[str] | Omit = omit,
        bot_category: List[
            Literal[
                "SEARCH_ENGINE_CRAWLER",
                "SEARCH_ENGINE_OPTIMIZATION",
                "MONITORING_AND_ANALYTICS",
                "ADVERTISING_AND_MARKETING",
                "SOCIAL_MEDIA_MARKETING",
                "PAGE_PREVIEW",
                "ACADEMIC_RESEARCH",
                "SECURITY",
                "ACCESSIBILITY",
                "WEBHOOKS",
                "FEED_FETCHER",
                "AI_CRAWLER",
                "AGGREGATOR",
                "AI_ASSISTANT",
                "AI_SEARCH",
                "ARCHIVER",
            ]
        ]
        | Omit = omit,
        bot_kind: List[Literal["AGENT", "BOT"]] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        bot_verification_status: List[Literal["VERIFIED"]] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotSummaryResponse:
        """
        Retrieves an aggregated summary of bots HTTP requests grouped by the specified
        dimension.

        Args:
          dimension: Specifies the attribute by which to group the results.

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bot: Filters results by bot name.

          bot_category: Filters results by bot category.

          bot_kind: Filters results by bot kind.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status (Verified vs. Unverified).

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/bots/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "asn": asn,
                        "bot": bot,
                        "bot_category": bot_category,
                        "bot_kind": bot_kind,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    bot_summary_params.BotSummaryParams,
                ),
                post_parser=ResultWrapper[BotSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotSummaryResponse], ResultWrapper[BotSummaryResponse]),
        )

    async def timeseries(
        self,
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        bot: SequenceNotStr[str] | Omit = omit,
        bot_category: List[
            Literal[
                "SEARCH_ENGINE_CRAWLER",
                "SEARCH_ENGINE_OPTIMIZATION",
                "MONITORING_AND_ANALYTICS",
                "ADVERTISING_AND_MARKETING",
                "SOCIAL_MEDIA_MARKETING",
                "PAGE_PREVIEW",
                "ACADEMIC_RESEARCH",
                "SECURITY",
                "ACCESSIBILITY",
                "WEBHOOKS",
                "FEED_FETCHER",
                "AI_CRAWLER",
                "AGGREGATOR",
                "AI_ASSISTANT",
                "AI_SEARCH",
                "ARCHIVER",
            ]
        ]
        | Omit = omit,
        bot_kind: List[Literal["AGENT", "BOT"]] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        bot_verification_status: List[Literal["VERIFIED"]] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotTimeseriesResponse:
        """
        Retrieves bots HTTP request volume over time.

        Args:
          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bot: Filters results by bot name.

          bot_category: Filters results by bot category.

          bot_kind: Filters results by bot kind.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status (Verified vs. Unverified).

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/bots/timeseries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "bot": bot,
                        "bot_category": bot_category,
                        "bot_kind": bot_kind,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "name": name,
                    },
                    bot_timeseries_params.BotTimeseriesParams,
                ),
                post_parser=ResultWrapper[BotTimeseriesResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotTimeseriesResponse], ResultWrapper[BotTimeseriesResponse]),
        )

    async def timeseries_groups(
        self,
        dimension: Literal["BOT", "BOT_KIND", "BOT_OPERATOR", "BOT_CATEGORY"],
        *,
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        bot: SequenceNotStr[str] | Omit = omit,
        bot_category: List[
            Literal[
                "SEARCH_ENGINE_CRAWLER",
                "SEARCH_ENGINE_OPTIMIZATION",
                "MONITORING_AND_ANALYTICS",
                "ADVERTISING_AND_MARKETING",
                "SOCIAL_MEDIA_MARKETING",
                "PAGE_PREVIEW",
                "ACADEMIC_RESEARCH",
                "SECURITY",
                "ACCESSIBILITY",
                "WEBHOOKS",
                "FEED_FETCHER",
                "AI_CRAWLER",
                "AGGREGATOR",
                "AI_ASSISTANT",
                "AI_SEARCH",
                "ARCHIVER",
            ]
        ]
        | Omit = omit,
        bot_kind: List[Literal["AGENT", "BOT"]] | Omit = omit,
        bot_operator: SequenceNotStr[str] | Omit = omit,
        bot_verification_status: List[Literal["VERIFIED"]] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        limit_per_group: int | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BotTimeseriesGroupsResponse:
        """
        Retrieves the distribution of HTTP requests from bots, grouped by the specified
        dimension over time.

        Args:
          dimension: Specifies the attribute by which to group the results.

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          bot: Filters results by bot name.

          bot_category: Filters results by bot category.

          bot_kind: Filters results by bot kind.

          bot_operator: Filters results by bot operator.

          bot_verification_status: Filters results by bot verification status (Verified vs. Unverified).

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          limit_per_group: Limits the number of objects per group to the top items within the specified
              time range. When item count exceeds the limit, extra items appear grouped under
              an "other" category.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/bots/timeseries_groups/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "bot": bot,
                        "bot_category": bot_category,
                        "bot_kind": bot_kind,
                        "bot_operator": bot_operator,
                        "bot_verification_status": bot_verification_status,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "limit_per_group": limit_per_group,
                        "location": location,
                        "name": name,
                    },
                    bot_timeseries_groups_params.BotTimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[BotTimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[BotTimeseriesGroupsResponse], ResultWrapper[BotTimeseriesGroupsResponse]),
        )


class BotsResourceWithRawResponse:
    def __init__(self, bots: BotsResource) -> None:
        self._bots = bots

        self.list = to_raw_response_wrapper(
            bots.list,
        )
        self.get = to_raw_response_wrapper(
            bots.get,
        )
        self.summary = to_raw_response_wrapper(
            bots.summary,
        )
        self.timeseries = to_raw_response_wrapper(
            bots.timeseries,
        )
        self.timeseries_groups = to_raw_response_wrapper(
            bots.timeseries_groups,
        )

    @cached_property
    def web_crawlers(self) -> WebCrawlersResourceWithRawResponse:
        return WebCrawlersResourceWithRawResponse(self._bots.web_crawlers)


class AsyncBotsResourceWithRawResponse:
    def __init__(self, bots: AsyncBotsResource) -> None:
        self._bots = bots

        self.list = async_to_raw_response_wrapper(
            bots.list,
        )
        self.get = async_to_raw_response_wrapper(
            bots.get,
        )
        self.summary = async_to_raw_response_wrapper(
            bots.summary,
        )
        self.timeseries = async_to_raw_response_wrapper(
            bots.timeseries,
        )
        self.timeseries_groups = async_to_raw_response_wrapper(
            bots.timeseries_groups,
        )

    @cached_property
    def web_crawlers(self) -> AsyncWebCrawlersResourceWithRawResponse:
        return AsyncWebCrawlersResourceWithRawResponse(self._bots.web_crawlers)


class BotsResourceWithStreamingResponse:
    def __init__(self, bots: BotsResource) -> None:
        self._bots = bots

        self.list = to_streamed_response_wrapper(
            bots.list,
        )
        self.get = to_streamed_response_wrapper(
            bots.get,
        )
        self.summary = to_streamed_response_wrapper(
            bots.summary,
        )
        self.timeseries = to_streamed_response_wrapper(
            bots.timeseries,
        )
        self.timeseries_groups = to_streamed_response_wrapper(
            bots.timeseries_groups,
        )

    @cached_property
    def web_crawlers(self) -> WebCrawlersResourceWithStreamingResponse:
        return WebCrawlersResourceWithStreamingResponse(self._bots.web_crawlers)


class AsyncBotsResourceWithStreamingResponse:
    def __init__(self, bots: AsyncBotsResource) -> None:
        self._bots = bots

        self.list = async_to_streamed_response_wrapper(
            bots.list,
        )
        self.get = async_to_streamed_response_wrapper(
            bots.get,
        )
        self.summary = async_to_streamed_response_wrapper(
            bots.summary,
        )
        self.timeseries = async_to_streamed_response_wrapper(
            bots.timeseries,
        )
        self.timeseries_groups = async_to_streamed_response_wrapper(
            bots.timeseries_groups,
        )

    @cached_property
    def web_crawlers(self) -> AsyncWebCrawlersResourceWithStreamingResponse:
        return AsyncWebCrawlersResourceWithStreamingResponse(self._bots.web_crawlers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .investigate_get_params import InvestigateGetParams as InvestigateGetParams
from .submission_list_params import SubmissionListParams as SubmissionListParams
from .investigate_list_params import InvestigateListParams as InvestigateListParams
from .investigate_get_response import InvestigateGetResponse as InvestigateGetResponse
from .submission_list_response import SubmissionListResponse as SubmissionListResponse
from .investigate_list_response import InvestigateListResponse as InvestigateListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BrowserCheck"]


class BrowserCheck(BaseModel):
    id: Optional[Literal["browser_check"]] = None
    """
    Inspect the visitor's browser for headers commonly associated with spammers and
    certain bots.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Browser Integrity Check."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LogoCreateParams"]


class LogoCreateParams(TypedDict, total=False):
    account_id: Required[str]

    image_data: Required[str]
    """Base64 encoded image data.

    Can include data URI prefix (e.g., 'data:image/png;base64,...') or just the
    base64 string.
    """

    similarity_threshold: Required[float]
    """Minimum similarity score (0-1) required for visual matches"""

    tag: Required[str]
    """Unique identifier for the logo query"""

    search_lookback: bool
    """
    If true, search historic scanned images for matches above the similarity
    threshold
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .logging import (
    LoggingResource,
    AsyncLoggingResource,
    LoggingResourceWithRawResponse,
    AsyncLoggingResourceWithRawResponse,
    LoggingResourceWithStreamingResponse,
    AsyncLoggingResourceWithStreamingResponse,
)
from .pacfiles import (
    PacfilesResource,
    AsyncPacfilesResource,
    PacfilesResourceWithRawResponse,
    AsyncPacfilesResourceWithRawResponse,
    PacfilesResourceWithStreamingResponse,
    AsyncPacfilesResourceWithStreamingResponse,
)
from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from .app_types import (
    AppTypesResource,
    AsyncAppTypesResource,
    AppTypesResourceWithRawResponse,
    AsyncAppTypesResourceWithRawResponse,
    AppTypesResourceWithStreamingResponse,
    AsyncAppTypesResourceWithStreamingResponse,
)
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from .lists.lists import (
    ListsResource,
    AsyncListsResource,
    ListsResourceWithRawResponse,
    AsyncListsResourceWithRawResponse,
    ListsResourceWithStreamingResponse,
    AsyncListsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from .certificates import (
    CertificatesResource,
    AsyncCertificatesResource,
    CertificatesResourceWithRawResponse,
    AsyncCertificatesResourceWithRawResponse,
    CertificatesResourceWithStreamingResponse,
    AsyncCertificatesResourceWithStreamingResponse,
)
from ...._base_client import make_request_options
from .proxy_endpoints import (
    ProxyEndpointsResource,
    AsyncProxyEndpointsResource,
    ProxyEndpointsResourceWithRawResponse,
    AsyncProxyEndpointsResourceWithRawResponse,
    ProxyEndpointsResourceWithStreamingResponse,
    AsyncProxyEndpointsResourceWithStreamingResponse,
)
from .audit_ssh_settings import (
    AuditSSHSettingsResource,
    AsyncAuditSSHSettingsResource,
    AuditSSHSettingsResourceWithRawResponse,
    AsyncAuditSSHSettingsResourceWithRawResponse,
    AuditSSHSettingsResourceWithStreamingResponse,
    AsyncAuditSSHSettingsResourceWithStreamingResponse,
)
from .configurations.configurations import (
    ConfigurationsResource,
    AsyncConfigurationsResource,
    ConfigurationsResourceWithRawResponse,
    AsyncConfigurationsResourceWithRawResponse,
    ConfigurationsResourceWithStreamingResponse,
    AsyncConfigurationsResourceWithStreamingResponse,
)
from ....types.zero_trust.gateway_list_response import GatewayListResponse
from ....types.zero_trust.gateway_create_response import GatewayCreateResponse

__all__ = ["GatewayResource", "AsyncGatewayResource"]


class GatewayResource(SyncAPIResource):
    @cached_property
    def audit_ssh_settings(self) -> AuditSSHSettingsResource:
        return AuditSSHSettingsResource(self._client)

    @cached_property
    def categories(self) -> CategoriesResource:
        return CategoriesResource(self._client)

    @cached_property
    def app_types(self) -> AppTypesResource:
        return AppTypesResource(self._client)

    @cached_property
    def configurations(self) -> ConfigurationsResource:
        return ConfigurationsResource(self._client)

    @cached_property
    def lists(self) -> ListsResource:
        return ListsResource(self._client)

    @cached_property
    def locations(self) -> LocationsResource:
        return LocationsResource(self._client)

    @cached_property
    def logging(self) -> LoggingResource:
        return LoggingResource(self._client)

    @cached_property
    def proxy_endpoints(self) -> ProxyEndpointsResource:
        return ProxyEndpointsResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def certificates(self) -> CertificatesResource:
        return CertificatesResource(self._client)

    @cached_property
    def pacfiles(self) -> PacfilesResource:
        return PacfilesResource(self._client)

    @cached_property
    def with_raw_response(self) -> GatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return GatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return GatewayResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayCreateResponse]:
        """
        Create a Zero Trust account for an existing Cloudflare account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/gateway", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayCreateResponse]], ResultWrapper[GatewayCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayListResponse]:
        """
        Retrieve information about the current Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/gateway", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayListResponse]], ResultWrapper[GatewayListResponse]),
        )


class AsyncGatewayResource(AsyncAPIResource):
    @cached_property
    def audit_ssh_settings(self) -> AsyncAuditSSHSettingsResource:
        return AsyncAuditSSHSettingsResource(self._client)

    @cached_property
    def categories(self) -> AsyncCategoriesResource:
        return AsyncCategoriesResource(self._client)

    @cached_property
    def app_types(self) -> AsyncAppTypesResource:
        return AsyncAppTypesResource(self._client)

    @cached_property
    def configurations(self) -> AsyncConfigurationsResource:
        return AsyncConfigurationsResource(self._client)

    @cached_property
    def lists(self) -> AsyncListsResource:
        return AsyncListsResource(self._client)

    @cached_property
    def locations(self) -> AsyncLocationsResource:
        return AsyncLocationsResource(self._client)

    @cached_property
    def logging(self) -> AsyncLoggingResource:
        return AsyncLoggingResource(self._client)

    @cached_property
    def proxy_endpoints(self) -> AsyncProxyEndpointsResource:
        return AsyncProxyEndpointsResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def certificates(self) -> AsyncCertificatesResource:
        return AsyncCertificatesResource(self._client)

    @cached_property
    def pacfiles(self) -> AsyncPacfilesResource:
        return AsyncPacfilesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncGatewayResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayCreateResponse]:
        """
        Create a Zero Trust account for an existing Cloudflare account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/gateway", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayCreateResponse]], ResultWrapper[GatewayCreateResponse]),
        )

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[GatewayListResponse]:
        """
        Retrieve information about the current Zero Trust account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/gateway", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[GatewayListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[GatewayListResponse]], ResultWrapper[GatewayListResponse]),
        )


class GatewayResourceWithRawResponse:
    def __init__(self, gateway: GatewayResource) -> None:
        self._gateway = gateway

        self.create = to_raw_response_wrapper(
            gateway.create,
        )
        self.list = to_raw_response_wrapper(
            gateway.list,
        )

    @cached_property
    def audit_ssh_settings(self) -> AuditSSHSettingsResourceWithRawResponse:
        return AuditSSHSettingsResourceWithRawResponse(self._gateway.audit_ssh_settings)

    @cached_property
    def categories(self) -> CategoriesResourceWithRawResponse:
        return CategoriesResourceWithRawResponse(self._gateway.categories)

    @cached_property
    def app_types(self) -> AppTypesResourceWithRawResponse:
        return AppTypesResourceWithRawResponse(self._gateway.app_types)

    @cached_property
    def configurations(self) -> ConfigurationsResourceWithRawResponse:
        return ConfigurationsResourceWithRawResponse(self._gateway.configurations)

    @cached_property
    def lists(self) -> ListsResourceWithRawResponse:
        return ListsResourceWithRawResponse(self._gateway.lists)

    @cached_property
    def locations(self) -> LocationsResourceWithRawResponse:
        return LocationsResourceWithRawResponse(self._gateway.locations)

    @cached_property
    def logging(self) -> LoggingResourceWithRawResponse:
        return LoggingResourceWithRawResponse(self._gateway.logging)

    @cached_property
    def proxy_endpoints(self) -> ProxyEndpointsResourceWithRawResponse:
        return ProxyEndpointsResourceWithRawResponse(self._gateway.proxy_endpoints)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._gateway.rules)

    @cached_property
    def certificates(self) -> CertificatesResourceWithRawResponse:
        return CertificatesResourceWithRawResponse(self._gateway.certificates)

    @cached_property
    def pacfiles(self) -> PacfilesResourceWithRawResponse:
        return PacfilesResourceWithRawResponse(self._gateway.pacfiles)


class AsyncGatewayResourceWithRawResponse:
    def __init__(self, gateway: AsyncGatewayResource) -> None:
        self._gateway = gateway

        self.create = async_to_raw_response_wrapper(
            gateway.create,
        )
        self.list = async_to_raw_response_wrapper(
            gateway.list,
        )

    @cached_property
    def audit_ssh_settings(self) -> AsyncAuditSSHSettingsResourceWithRawResponse:
        return AsyncAuditSSHSettingsResourceWithRawResponse(self._gateway.audit_ssh_settings)

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithRawResponse:
        return AsyncCategoriesResourceWithRawResponse(self._gateway.categories)

    @cached_property
    def app_types(self) -> AsyncAppTypesResourceWithRawResponse:
        return AsyncAppTypesResourceWithRawResponse(self._gateway.app_types)

    @cached_property
    def configurations(self) -> AsyncConfigurationsResourceWithRawResponse:
        return AsyncConfigurationsResourceWithRawResponse(self._gateway.configurations)

    @cached_property
    def lists(self) -> AsyncListsResourceWithRawResponse:
        return AsyncListsResourceWithRawResponse(self._gateway.lists)

    @cached_property
    def locations(self) -> AsyncLocationsResourceWithRawResponse:
        return AsyncLocationsResourceWithRawResponse(self._gateway.locations)

    @cached_property
    def logging(self) -> AsyncLoggingResourceWithRawResponse:
        return AsyncLoggingResourceWithRawResponse(self._gateway.logging)

    @cached_property
    def proxy_endpoints(self) -> AsyncProxyEndpointsResourceWithRawResponse:
        return AsyncProxyEndpointsResourceWithRawResponse(self._gateway.proxy_endpoints)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._gateway.rules)

    @cached_property
    def certificates(self) -> AsyncCertificatesResourceWithRawResponse:
        return AsyncCertificatesResourceWithRawResponse(self._gateway.certificates)

    @cached_property
    def pacfiles(self) -> AsyncPacfilesResourceWithRawResponse:
        return AsyncPacfilesResourceWithRawResponse(self._gateway.pacfiles)


class GatewayResourceWithStreamingResponse:
    def __init__(self, gateway: GatewayResource) -> None:
        self._gateway = gateway

        self.create = to_streamed_response_wrapper(
            gateway.create,
        )
        self.list = to_streamed_response_wrapper(
            gateway.list,
        )

    @cached_property
    def audit_ssh_settings(self) -> AuditSSHSettingsResourceWithStreamingResponse:
        return AuditSSHSettingsResourceWithStreamingResponse(self._gateway.audit_ssh_settings)

    @cached_property
    def categories(self) -> CategoriesResourceWithStreamingResponse:
        return CategoriesResourceWithStreamingResponse(self._gateway.categories)

    @cached_property
    def app_types(self) -> AppTypesResourceWithStreamingResponse:
        return AppTypesResourceWithStreamingResponse(self._gateway.app_types)

    @cached_property
    def configurations(self) -> ConfigurationsResourceWithStreamingResponse:
        return ConfigurationsResourceWithStreamingResponse(self._gateway.configurations)

    @cached_property
    def lists(self) -> ListsResourceWithStreamingResponse:
        return ListsResourceWithStreamingResponse(self._gateway.lists)

    @cached_property
    def locations(self) -> LocationsResourceWithStreamingResponse:
        return LocationsResourceWithStreamingResponse(self._gateway.locations)

    @cached_property
    def logging(self) -> LoggingResourceWithStreamingResponse:
        return LoggingResourceWithStreamingResponse(self._gateway.logging)

    @cached_property
    def proxy_endpoints(self) -> ProxyEndpointsResourceWithStreamingResponse:
        return ProxyEndpointsResourceWithStreamingResponse(self._gateway.proxy_endpoints)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._gateway.rules)

    @cached_property
    def certificates(self) -> CertificatesResourceWithStreamingResponse:
        return CertificatesResourceWithStreamingResponse(self._gateway.certificates)

    @cached_property
    def pacfiles(self) -> PacfilesResourceWithStreamingResponse:
        return PacfilesResourceWithStreamingResponse(self._gateway.pacfiles)


class AsyncGatewayResourceWithStreamingResponse:
    def __init__(self, gateway: AsyncGatewayResource) -> None:
        self._gateway = gateway

        self.create = async_to_streamed_response_wrapper(
            gateway.create,
        )
        self.list = async_to_streamed_response_wrapper(
            gateway.list,
        )

    @cached_property
    def audit_ssh_settings(self) -> AsyncAuditSSHSettingsResourceWithStreamingResponse:
        return AsyncAuditSSHSettingsResourceWithStreamingResponse(self._gateway.audit_ssh_settings)

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithStreamingResponse:
        return AsyncCategoriesResourceWithStreamingResponse(self._gateway.categories)

    @cached_property
    def app_types(self) -> AsyncAppTypesResourceWithStreamingResponse:
        return AsyncAppTypesResourceWithStreamingResponse(self._gateway.app_types)

    @cached_property
    def configurations(self) -> AsyncConfigurationsResourceWithStreamingResponse:
        return AsyncConfigurationsResourceWithStreamingResponse(self._gateway.configurations)

    @cached_property
    def lists(self) -> AsyncListsResourceWithStreamingResponse:
        return AsyncListsResourceWithStreamingResponse(self._gateway.lists)

    @cached_property
    def locations(self) -> AsyncLocationsResourceWithStreamingResponse:
        return AsyncLocationsResourceWithStreamingResponse(self._gateway.locations)

    @cached_property
    def logging(self) -> AsyncLoggingResourceWithStreamingResponse:
        return AsyncLoggingResourceWithStreamingResponse(self._gateway.logging)

    @cached_property
    def proxy_endpoints(self) -> AsyncProxyEndpointsResourceWithStreamingResponse:
        return AsyncProxyEndpointsResourceWithStreamingResponse(self._gateway.proxy_endpoints)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._gateway.rules)

    @cached_property
    def certificates(self) -> AsyncCertificatesResourceWithStreamingResponse:
        return AsyncCertificatesResourceWithStreamingResponse(self._gateway.certificates)

    @cached_property
    def pacfiles(self) -> AsyncPacfilesResourceWithStreamingResponse:
        return AsyncPacfilesResourceWithStreamingResponse(self._gateway.pacfiles)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .spam import (
    SpamResource,
    AsyncSpamResource,
    SpamResourceWithRawResponse,
    AsyncSpamResourceWithRawResponse,
    SpamResourceWithStreamingResponse,
    AsyncSpamResourceWithStreamingResponse,
)
from .tlds import (
    TLDsResource,
    AsyncTLDsResource,
    TLDsResourceWithRawResponse,
    AsyncTLDsResourceWithRawResponse,
    TLDsResourceWithStreamingResponse,
    AsyncTLDsResourceWithStreamingResponse,
)
from .spoof import (
    SpoofResource,
    AsyncSpoofResource,
    SpoofResourceWithRawResponse,
    AsyncSpoofResourceWithRawResponse,
    SpoofResourceWithStreamingResponse,
    AsyncSpoofResourceWithStreamingResponse,
)
from .malicious import (
    MaliciousResource,
    AsyncMaliciousResource,
    MaliciousResourceWithRawResponse,
    AsyncMaliciousResourceWithRawResponse,
    MaliciousResourceWithStreamingResponse,
    AsyncMaliciousResourceWithStreamingResponse,
)

__all__ = [
    "MaliciousResource",
    "AsyncMaliciousResource",
    "MaliciousResourceWithRawResponse",
    "AsyncMaliciousResourceWithRawResponse",
    "MaliciousResourceWithStreamingResponse",
    "AsyncMaliciousResourceWithStreamingResponse",
    "SpamResource",
    "AsyncSpamResource",
    "SpamResourceWithRawResponse",
    "AsyncSpamResourceWithRawResponse",
    "SpamResourceWithStreamingResponse",
    "AsyncSpamResourceWithStreamingResponse",
    "SpoofResource",
    "AsyncSpoofResource",
    "SpoofResourceWithRawResponse",
    "AsyncSpoofResourceWithRawResponse",
    "SpoofResourceWithStreamingResponse",
    "AsyncSpoofResourceWithStreamingResponse",
    "TLDsResource",
    "AsyncTLDsResource",
    "TLDsResourceWithRawResponse",
    "AsyncTLDsResourceWithRawResponse",
    "TLDsResourceWithStreamingResponse",
    "AsyncTLDsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "Configuration",
    "AuthIDCharacteristic",
    "AuthIDCharacteristicAPIShieldAuthIDCharacteristic",
    "AuthIDCharacteristicAPIShieldAuthIDCharacteristicJWTClaim",
]


class AuthIDCharacteristicAPIShieldAuthIDCharacteristic(BaseModel):
    """Auth ID Characteristic"""

    name: str
    """The name of the characteristic field, i.e., the header or cookie name."""

    type: Literal["header", "cookie"]
    """The type of characteristic."""


class AuthIDCharacteristicAPIShieldAuthIDCharacteristicJWTClaim(BaseModel):
    """Auth ID Characteristic extracted from JWT Token Claims"""

    name: str
    """
    Claim location expressed as `$(token_config_id):$(json_path)`, where
    `token_config_id` is the ID of the token configuration used in validating the
    JWT, and `json_path` is a RFC 9535 JSONPath
    (https://goessner.net/articles/JsonPath/,
    https://www.rfc-editor.org/rfc/rfc9535.html). The JSONPath expression may be in
    dot or bracket notation, may only specify literal keys or array indexes, and
    must return a singleton value, which will be interpreted as a string.
    """

    type: Literal["jwt"]
    """The type of characteristic."""


AuthIDCharacteristic: TypeAlias = Union[
    AuthIDCharacteristicAPIShieldAuthIDCharacteristic, AuthIDCharacteristicAPIShieldAuthIDCharacteristicJWTClaim
]


class Configuration(BaseModel):
    auth_id_characteristics: List[AuthIDCharacteristic]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["HostnameUpdateParams", "Config"]


class HostnameUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    config: Required[Iterable[Config]]


class Config(TypedDict, total=False):
    cert_id: str
    """Certificate identifier tag."""

    enabled: Optional[bool]
    """Indicates whether hostname-level authenticated origin pulls is enabled.

    A null value voids the association.
    """

    hostname: str
    """
    The hostname on the origin for which the client certificate uploaded will be
    used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cors import (
    CORSResource,
    AsyncCORSResource,
    CORSResourceWithRawResponse,
    AsyncCORSResourceWithRawResponse,
    CORSResourceWithStreamingResponse,
    AsyncCORSResourceWithStreamingResponse,
)
from .locks import (
    LocksResource,
    AsyncLocksResource,
    LocksResourceWithRawResponse,
    AsyncLocksResourceWithRawResponse,
    LocksResourceWithStreamingResponse,
    AsyncLocksResourceWithStreamingResponse,
)
from .sippy import (
    SippyResource,
    AsyncSippyResource,
    SippyResourceWithRawResponse,
    AsyncSippyResourceWithRawResponse,
    SippyResourceWithStreamingResponse,
    AsyncSippyResourceWithStreamingResponse,
)
from .buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from .domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from .metrics import (
    MetricsResource,
    AsyncMetricsResource,
    MetricsResourceWithRawResponse,
    AsyncMetricsResourceWithRawResponse,
    MetricsResourceWithStreamingResponse,
    AsyncMetricsResourceWithStreamingResponse,
)
from .objects import (
    ObjectsResource,
    AsyncObjectsResource,
    ObjectsResourceWithRawResponse,
    AsyncObjectsResourceWithRawResponse,
    ObjectsResourceWithStreamingResponse,
    AsyncObjectsResourceWithStreamingResponse,
)
from .lifecycle import (
    LifecycleResource,
    AsyncLifecycleResource,
    LifecycleResourceWithRawResponse,
    AsyncLifecycleResourceWithRawResponse,
    LifecycleResourceWithStreamingResponse,
    AsyncLifecycleResourceWithStreamingResponse,
)
from .event_notifications import (
    EventNotificationsResource,
    AsyncEventNotificationsResource,
    EventNotificationsResourceWithRawResponse,
    AsyncEventNotificationsResourceWithRawResponse,
    EventNotificationsResourceWithStreamingResponse,
    AsyncEventNotificationsResourceWithStreamingResponse,
)

__all__ = [
    "LifecycleResource",
    "AsyncLifecycleResource",
    "LifecycleResourceWithRawResponse",
    "AsyncLifecycleResourceWithRawResponse",
    "LifecycleResourceWithStreamingResponse",
    "AsyncLifecycleResourceWithStreamingResponse",
    "CORSResource",
    "AsyncCORSResource",
    "CORSResourceWithRawResponse",
    "AsyncCORSResourceWithRawResponse",
    "CORSResourceWithStreamingResponse",
    "AsyncCORSResourceWithStreamingResponse",
    "DomainsResource",
    "AsyncDomainsResource",
    "DomainsResourceWithRawResponse",
    "AsyncDomainsResourceWithRawResponse",
    "DomainsResourceWithStreamingResponse",
    "AsyncDomainsResourceWithStreamingResponse",
    "EventNotificationsResource",
    "AsyncEventNotificationsResource",
    "EventNotificationsResourceWithRawResponse",
    "AsyncEventNotificationsResourceWithRawResponse",
    "EventNotificationsResourceWithStreamingResponse",
    "AsyncEventNotificationsResourceWithStreamingResponse",
    "LocksResource",
    "AsyncLocksResource",
    "LocksResourceWithRawResponse",
    "AsyncLocksResourceWithRawResponse",
    "LocksResourceWithStreamingResponse",
    "AsyncLocksResourceWithStreamingResponse",
    "MetricsResource",
    "AsyncMetricsResource",
    "MetricsResourceWithRawResponse",
    "AsyncMetricsResourceWithRawResponse",
    "MetricsResourceWithStreamingResponse",
    "AsyncMetricsResourceWithStreamingResponse",
    "SippyResource",
    "AsyncSippyResource",
    "SippyResourceWithRawResponse",
    "AsyncSippyResourceWithRawResponse",
    "SippyResourceWithStreamingResponse",
    "AsyncSippyResourceWithStreamingResponse",
    "ObjectsResource",
    "AsyncObjectsResource",
    "ObjectsResourceWithRawResponse",
    "AsyncObjectsResourceWithRawResponse",
    "ObjectsResourceWithStreamingResponse",
    "AsyncObjectsResourceWithStreamingResponse",
    "BucketsResource",
    "AsyncBucketsResource",
    "BucketsResourceWithRawResponse",
    "AsyncBucketsResourceWithRawResponse",
    "BucketsResourceWithStreamingResponse",
    "AsyncBucketsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["Mechanism", "Email", "Pagerduty", "Webhook"]


class Email(BaseModel):
    id: Optional[str] = None
    """The email address"""


class Pagerduty(BaseModel):
    id: Optional[str] = None
    """UUID"""


class Webhook(BaseModel):
    id: Optional[str] = None
    """UUID"""


class Mechanism(BaseModel):
    """List of IDs that will be used when dispatching a notification.

    IDs for email type will be the email address.
    """

    email: Optional[List[Email]] = None

    pagerduty: Optional[List[Pagerduty]] = None

    webhooks: Optional[List[Webhook]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CrowdstrikeInput"]


class CrowdstrikeInput(BaseModel):
    connection_id: str
    """Posture Integration ID."""

    last_seen: Optional[str] = None
    """For more details on last seen, please refer to the Crowdstrike documentation."""

    operator: Optional[Literal["<", "<=", ">", ">=", "=="]] = None
    """Operator."""

    os: Optional[str] = None
    """Os Version."""

    overall: Optional[str] = None
    """Overall."""

    sensor_config: Optional[str] = None
    """SensorConfig."""

    state: Optional[Literal["online", "offline", "unknown"]] = None
    """For more details on state, please refer to the Crowdstrike documentation."""

    version: Optional[str] = None
    """Version."""

    version_operator: Optional[Literal["<", "<=", ">", ">=", "=="]] = FieldInfo(alias="versionOperator", default=None)
    """Version Operator."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ActiveSessionKickAllParticipantsResponse", "Data"]


class Data(BaseModel):
    action: Optional[str] = None

    kicked_participants_count: Optional[float] = None


class ActiveSessionKickAllParticipantsResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CloudIntegrationListParams"]


class CloudIntegrationListParams(TypedDict, total=False):
    account_id: Required[str]

    cloudflare: bool

    desc: bool

    order_by: str
    """One of ["updated_at", "id", "cloud_type", "name"]."""

    status: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel
from .issue_type import IssueType

__all__ = ["IssueListResponse", "Issue", "IssuePayload"]


class IssuePayload(BaseModel):
    detection_method: Optional[str] = None
    """Describes the method used to detect insight."""

    zone_tag: Optional[str] = None


class Issue(BaseModel):
    id: Optional[str] = None

    dismissed: Optional[bool] = None

    has_extended_context: Optional[bool] = None
    """
    Indicates whether the insight has a large payload that requires fetching via the
    context endpoint.
    """

    issue_class: Optional[str] = None

    issue_type: Optional[IssueType] = None

    payload: Optional[IssuePayload] = None

    resolve_link: Optional[str] = None

    resolve_text: Optional[str] = None

    severity: Optional[Literal["Low", "Moderate", "Critical"]] = None

    since: Optional[datetime] = None

    status: Optional[Literal["active", "resolved"]] = None
    """The current status of the insight."""

    subject: Optional[str] = None

    timestamp: Optional[datetime] = None

    user_classification: Optional[Literal["false_positive", "accept_risk", "other"]] = None
    """User-defined classification for the insight.

    Can be 'false_positive', 'accept_risk', 'other', or null.
    """


class IssueListResponse(BaseModel):
    count: Optional[int] = None
    """Indicates the total number of results."""

    issues: Optional[List[Issue]] = None

    page: Optional[int] = None
    """Specifies the current page within paginated list of results."""

    per_page: Optional[int] = None
    """Sets the number of results per page of results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["Advertisement"]


class Advertisement(BaseModel):
    automatic_advertisement: Optional[bool] = None
    """
    Toggle on if you would like Cloudflare to automatically advertise the IP
    Prefixes within the rule via Magic Transit when the rule is triggered. Only
    available for users of Magic Transit.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .notification_settings import NotificationSettings

__all__ = ["AntiVirusSettings"]


class AntiVirusSettings(BaseModel):
    """Specify anti-virus settings."""

    enabled_download_phase: Optional[bool] = None
    """Specify whether to enable anti-virus scanning on downloads."""

    enabled_upload_phase: Optional[bool] = None
    """Specify whether to enable anti-virus scanning on uploads."""

    fail_closed: Optional[bool] = None
    """Specify whether to block requests for unscannable files."""

    notification_settings: Optional[NotificationSettings] = None
    """Configure the message the user's device shows during an antivirus scan."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.workers.scripts import tail_create_params
from ....types.workers.scripts.tail_get_response import TailGetResponse
from ....types.workers.scripts.tail_create_response import TailCreateResponse
from ....types.workers.scripts.tail_delete_response import TailDeleteResponse

__all__ = ["TailResource", "AsyncTailResource"]


class TailResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TailResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TailResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TailResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TailResourceWithStreamingResponse(self)

    def create(
        self,
        script_name: str,
        *,
        account_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TailCreateResponse:
        """
        Starts a tail that receives logs and exception from a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/tails",
                account_id=account_id,
                script_name=script_name,
            ),
            body=maybe_transform(body, tail_create_params.TailCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TailCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TailCreateResponse], ResultWrapper[TailCreateResponse]),
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TailDeleteResponse:
        """
        Deletes a tail from a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/tails/{id}",
                account_id=account_id,
                script_name=script_name,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TailDeleteResponse,
        )

    def get(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TailGetResponse:
        """
        Get list of tails currently deployed on a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/tails",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TailGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[TailGetResponse], ResultWrapper[TailGetResponse]),
        )


class AsyncTailResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTailResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTailResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTailResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTailResourceWithStreamingResponse(self)

    async def create(
        self,
        script_name: str,
        *,
        account_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TailCreateResponse:
        """
        Starts a tail that receives logs and exception from a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/tails",
                account_id=account_id,
                script_name=script_name,
            ),
            body=await async_maybe_transform(body, tail_create_params.TailCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TailCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TailCreateResponse], ResultWrapper[TailCreateResponse]),
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TailDeleteResponse:
        """
        Deletes a tail from a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/tails/{id}",
                account_id=account_id,
                script_name=script_name,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TailDeleteResponse,
        )

    async def get(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TailGetResponse:
        """
        Get list of tails currently deployed on a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/tails",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TailGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[TailGetResponse], ResultWrapper[TailGetResponse]),
        )


class TailResourceWithRawResponse:
    def __init__(self, tail: TailResource) -> None:
        self._tail = tail

        self.create = to_raw_response_wrapper(
            tail.create,
        )
        self.delete = to_raw_response_wrapper(
            tail.delete,
        )
        self.get = to_raw_response_wrapper(
            tail.get,
        )


class AsyncTailResourceWithRawResponse:
    def __init__(self, tail: AsyncTailResource) -> None:
        self._tail = tail

        self.create = async_to_raw_response_wrapper(
            tail.create,
        )
        self.delete = async_to_raw_response_wrapper(
            tail.delete,
        )
        self.get = async_to_raw_response_wrapper(
            tail.get,
        )


class TailResourceWithStreamingResponse:
    def __init__(self, tail: TailResource) -> None:
        self._tail = tail

        self.create = to_streamed_response_wrapper(
            tail.create,
        )
        self.delete = to_streamed_response_wrapper(
            tail.delete,
        )
        self.get = to_streamed_response_wrapper(
            tail.get,
        )


class AsyncTailResourceWithStreamingResponse:
    def __init__(self, tail: AsyncTailResource) -> None:
        self._tail = tail

        self.create = async_to_streamed_response_wrapper(
            tail.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            tail.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            tail.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.cloudforce_one.threat_events import attacker_list_params
from ....types.cloudforce_one.threat_events.attacker_list_response import AttackerListResponse

__all__ = ["AttackersResource", "AsyncAttackersResource"]


class AttackersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AttackersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AttackersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AttackersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AttackersResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AttackerListResponse:
        """
        Lists attackers across multiple datasets

        Args:
          account_id: Account ID.

          dataset_ids: Array of dataset IDs to query attackers from. If not provided, uses the default
              dataset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/attackers", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"dataset_ids": dataset_ids}, attacker_list_params.AttackerListParams),
            ),
            cast_to=AttackerListResponse,
        )


class AsyncAttackersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAttackersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAttackersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAttackersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAttackersResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AttackerListResponse:
        """
        Lists attackers across multiple datasets

        Args:
          account_id: Account ID.

          dataset_ids: Array of dataset IDs to query attackers from. If not provided, uses the default
              dataset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/attackers", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"dataset_ids": dataset_ids}, attacker_list_params.AttackerListParams
                ),
            ),
            cast_to=AttackerListResponse,
        )


class AttackersResourceWithRawResponse:
    def __init__(self, attackers: AttackersResource) -> None:
        self._attackers = attackers

        self.list = to_raw_response_wrapper(
            attackers.list,
        )


class AsyncAttackersResourceWithRawResponse:
    def __init__(self, attackers: AsyncAttackersResource) -> None:
        self._attackers = attackers

        self.list = async_to_raw_response_wrapper(
            attackers.list,
        )


class AttackersResourceWithStreamingResponse:
    def __init__(self, attackers: AttackersResource) -> None:
        self._attackers = attackers

        self.list = to_streamed_response_wrapper(
            attackers.list,
        )


class AsyncAttackersResourceWithStreamingResponse:
    def __init__(self, attackers: AsyncAttackersResource) -> None:
        self._attackers = attackers

        self.list = async_to_streamed_response_wrapper(
            attackers.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CacheLevelParam"]


class CacheLevelParam(TypedDict, total=False):
    id: Literal["cache_level"]
    """Apply custom caching based on the option selected."""

    value: Literal["bypass", "basic", "simplified", "aggressive", "cache_everything"]
    """
    - `bypass`: Cloudflare does not cache.
    - `basic`: Delivers resources from cache when there is no query string.
    - `simplified`: Delivers the same resource to everyone independent of the query
      string.
    - `aggressive`: Caches all static content that has a query string.
    - `cache_everything`: Treats all content as static and caches all file types
      beyond the
      [Cloudflare default cached content](https://developers.cloudflare.com/cache/concepts/default-cache-behavior/#default-cached-file-extensions).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .indexes import (
    IndexesResource,
    AsyncIndexesResource,
    IndexesResourceWithRawResponse,
    AsyncIndexesResourceWithRawResponse,
    IndexesResourceWithStreamingResponse,
    AsyncIndexesResourceWithStreamingResponse,
)
from .vectorize import (
    VectorizeResource,
    AsyncVectorizeResource,
    VectorizeResourceWithRawResponse,
    AsyncVectorizeResourceWithRawResponse,
    VectorizeResourceWithStreamingResponse,
    AsyncVectorizeResourceWithStreamingResponse,
)

__all__ = [
    "IndexesResource",
    "AsyncIndexesResource",
    "IndexesResourceWithRawResponse",
    "AsyncIndexesResourceWithRawResponse",
    "IndexesResourceWithStreamingResponse",
    "AsyncIndexesResourceWithStreamingResponse",
    "VectorizeResource",
    "AsyncVectorizeResource",
    "VectorizeResourceWithRawResponse",
    "AsyncVectorizeResourceWithRawResponse",
    "VectorizeResourceWithStreamingResponse",
    "AsyncVectorizeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .issues import (
    IssuesResource,
    AsyncIssuesResource,
    IssuesResourceWithRawResponse,
    AsyncIssuesResourceWithRawResponse,
    IssuesResourceWithStreamingResponse,
    AsyncIssuesResourceWithStreamingResponse,
)
from .issue_types import (
    IssueTypesResource,
    AsyncIssueTypesResource,
    IssueTypesResourceWithRawResponse,
    AsyncIssueTypesResourceWithRawResponse,
    IssueTypesResourceWithStreamingResponse,
    AsyncIssueTypesResourceWithStreamingResponse,
)
from .attack_surface_report import (
    AttackSurfaceReportResource,
    AsyncAttackSurfaceReportResource,
    AttackSurfaceReportResourceWithRawResponse,
    AsyncAttackSurfaceReportResourceWithRawResponse,
    AttackSurfaceReportResourceWithStreamingResponse,
    AsyncAttackSurfaceReportResourceWithStreamingResponse,
)

__all__ = [
    "IssueTypesResource",
    "AsyncIssueTypesResource",
    "IssueTypesResourceWithRawResponse",
    "AsyncIssueTypesResourceWithRawResponse",
    "IssueTypesResourceWithStreamingResponse",
    "AsyncIssueTypesResourceWithStreamingResponse",
    "IssuesResource",
    "AsyncIssuesResource",
    "IssuesResourceWithRawResponse",
    "AsyncIssuesResourceWithRawResponse",
    "IssuesResourceWithStreamingResponse",
    "AsyncIssuesResourceWithStreamingResponse",
    "AttackSurfaceReportResource",
    "AsyncAttackSurfaceReportResource",
    "AttackSurfaceReportResourceWithRawResponse",
    "AsyncAttackSurfaceReportResourceWithRawResponse",
    "AttackSurfaceReportResourceWithStreamingResponse",
    "AsyncAttackSurfaceReportResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["ClassificationUpdateParams"]


class ClassificationUpdateParams(TypedDict, total=False):
    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""

    classification: Optional[Literal["false_positive", "accept_risk", "other"]]
    """User-defined classification for the insight.

    Can be 'false_positive', 'accept_risk', 'other', or null.
    """

    rationale: str
    """Rationale for the classification change.

    Required when classification is 'accept_risk' or 'other'.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .consumer import Consumer
from ..._models import BaseModel

__all__ = ["Queue", "Producer", "ProducerMqWorkerProducer", "ProducerMqR2Producer", "Settings"]


class ProducerMqWorkerProducer(BaseModel):
    script: Optional[str] = None

    type: Optional[Literal["worker"]] = None


class ProducerMqR2Producer(BaseModel):
    bucket_name: Optional[str] = None

    type: Optional[Literal["r2_bucket"]] = None


Producer: TypeAlias = Union[ProducerMqWorkerProducer, ProducerMqR2Producer]


class Settings(BaseModel):
    delivery_delay: Optional[float] = None
    """Number of seconds to delay delivery of all messages to consumers."""

    delivery_paused: Optional[bool] = None
    """Indicates if message delivery to consumers is currently paused."""

    message_retention_period: Optional[float] = None
    """Number of seconds after which an unconsumed message will be delayed."""


class Queue(BaseModel):
    consumers: Optional[List[Consumer]] = None

    consumers_total_count: Optional[float] = None

    created_on: Optional[str] = None

    modified_on: Optional[str] = None

    producers: Optional[List[Producer]] = None

    producers_total_count: Optional[float] = None

    queue_id: Optional[str] = None

    queue_name: Optional[str] = None

    settings: Optional[Settings] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .host import Host
from ..._types import SequenceNotStr

__all__ = ["CertificatePackCreateParams"]


class CertificatePackCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    certificate_authority: Required[Literal["google", "lets_encrypt", "ssl_com"]]
    """Certificate Authority selected for the order.

    For information on any certificate authority specific details or restrictions
    [see this page for more details.](https://developers.cloudflare.com/ssl/reference/certificate-authorities)
    """

    hosts: Required[SequenceNotStr[Host]]
    """Comma separated list of valid host names for the certificate packs.

    Must contain the zone apex, may not contain more than 50 hosts, and may not be
    empty.
    """

    type: Required[Literal["advanced"]]
    """Type of certificate pack."""

    validation_method: Required[Literal["txt", "http", "email"]]
    """Validation Method selected for the order."""

    validity_days: Required[Literal[14, 30, 90, 365]]
    """Validity Days selected for the order."""

    cloudflare_branding: bool
    """Whether or not to add Cloudflare Branding for the order.

    This will add a subdomain of sni.cloudflaressl.com as the Common Name if set to
    true.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .custom import (
    CustomResource,
    AsyncCustomResource,
    CustomResourceWithRawResponse,
    AsyncCustomResourceWithRawResponse,
    CustomResourceWithStreamingResponse,
    AsyncCustomResourceWithStreamingResponse,
)
from .default import (
    DefaultResource,
    AsyncDefaultResource,
    DefaultResourceWithRawResponse,
    AsyncDefaultResourceWithRawResponse,
    DefaultResourceWithStreamingResponse,
    AsyncDefaultResourceWithStreamingResponse,
)
from .policies import (
    PoliciesResource,
    AsyncPoliciesResource,
    PoliciesResourceWithRawResponse,
    AsyncPoliciesResourceWithRawResponse,
    PoliciesResourceWithStreamingResponse,
    AsyncPoliciesResourceWithStreamingResponse,
)

__all__ = [
    "DefaultResource",
    "AsyncDefaultResource",
    "DefaultResourceWithRawResponse",
    "AsyncDefaultResourceWithRawResponse",
    "DefaultResourceWithStreamingResponse",
    "AsyncDefaultResourceWithStreamingResponse",
    "CustomResource",
    "AsyncCustomResource",
    "CustomResourceWithRawResponse",
    "AsyncCustomResourceWithRawResponse",
    "CustomResourceWithStreamingResponse",
    "AsyncCustomResourceWithStreamingResponse",
    "PoliciesResource",
    "AsyncPoliciesResource",
    "PoliciesResourceWithRawResponse",
    "AsyncPoliciesResourceWithRawResponse",
    "PoliciesResourceWithStreamingResponse",
    "AsyncPoliciesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["FleetStatusOverTimeParams"]


class FleetStatusOverTimeParams(TypedDict, total=False):
    account_id: Required[str]

    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """Time range beginning in ISO format"""

    to: Required[str]
    """Time range end in ISO format"""

    colo: str
    """Cloudflare colo"""

    device_id: str
    """Device-specific ID, given as UUID v4"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["Connection"]


class Connection(BaseModel):
    id: str
    """Identifier"""

    added_at: datetime

    first_seen_at: datetime

    host: str

    last_seen_at: datetime

    url: str

    url_contains_cdn_cgi_path: bool

    domain_reported_malicious: Optional[bool] = None

    first_page_url: Optional[str] = None

    malicious_domain_categories: Optional[List[str]] = None

    malicious_url_categories: Optional[List[str]] = None

    page_urls: Optional[List[str]] = None

    url_reported_malicious: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ContentScanningUpdateParams"]


class ContentScanningUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    value: Required[Literal["enabled", "disabled"]]
    """The status value for Content Scanning."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WebhookUpdateParams"]


class WebhookUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account id"""

    name: Required[str]
    """The name of the webhook destination.

    This will be included in the request body when you receive a webhook
    notification.
    """

    url: Required[str]
    """The POST endpoint to call when dispatching a notification."""

    secret: str
    """
    Optional secret that will be passed in the `cf-webhook-auth` header when
    dispatching generic webhook notifications or formatted for supported
    destinations. Secrets are not returned in any API response body.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SecretGetParams"]


class SecretGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    dispatch_namespace: Required[str]
    """Name of the Workers for Platforms dispatch namespace."""

    script_name: Required[str]
    """Name of the script, used in URLs and route configuration."""

    url_encoded: bool
    """Flag that indicates whether the secret name is URL encoded."""
